/*
 * Copyright 2022 Stéphane Caron
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "gtest/gtest.h"
#include "mpacklog/CircularBuffer.h"

namespace mpacklog {

TEST(CircularBuffer, PushAndPop) {
  CircularBuffer<double, 10> buffer;
  double popped;
  buffer.push(1.);
  buffer.push(2.);
  buffer.pop(popped);
  ASSERT_FALSE(buffer.empty());
  buffer.pop(popped);
  ASSERT_TRUE(buffer.empty());
}

TEST(CircularBuffer, CannotPushForever) {
  CircularBuffer<double, 3> buffer;
  ASSERT_TRUE(buffer.push(1.));
  ASSERT_TRUE(buffer.push(2.));
  ASSERT_TRUE(buffer.push(3.));
  ASSERT_FALSE(buffer.push(4.));
  ASSERT_FALSE(buffer.push(5.));

  double popped;
  ASSERT_TRUE(buffer.pop(popped));
  ASSERT_TRUE(buffer.pop(popped));
  ASSERT_TRUE(buffer.pop(popped));
  ASSERT_FALSE(buffer.pop(popped));
  ASSERT_FALSE(buffer.pop(popped));
}

}  // namespace mpacklog
